clear all
clc
%% Parameters can change based on the particular scenario
r=0.17;      % hospital admission rate

%% Screening individuals and related parameters
theta= 0.925;     % fraction of admitted from outside who are screened
sigma=0.0;        % fraction of in-hospital exposure to CD infection (under treatment) who are screened
bs=0.95;          % fraction of screened individuals being susceptible
alpha=0.22;       % fraction of screened individuals, susceptible, who receive antibiotics
eta=0.6;          % fraction of screened individuals, colonized, with immune protection [Kyne Et al(23)]
delta=0.000122;   % fraction of admitted individuals with CDI (symptomatic) 

%% Other parameters
gamma=1/5;      % recovery rate of infection
epsilon=1/5;    % rate of developing symptomatic infection (new vaccine paper)
q=0.2;          % fraction individuals in whom C. Diff treatment fails [McFarlancd(24)]
tauminus=0.11;  % rate of ceased antibiotic treatment for susceptible individuals (fixed)
tauplus=0.011;  % rate of antibiotic treatment for susceptible individuals
fplus=0.6;      % fraction of colonized individuals with immune protection
psi=0.2;        % reduction of susceptibility to colonization without antibiotic treatment
rho=1/4;        % rate of resolving symptoms for CDI under treatment

%% Discharge rate
mu=0.17;           % all except CDI
muI=0.0012;        % CDI

%% reduction of infectiousness
nu=0.5;         % due to immune protection
kappa=0.5;      % due to asymptomatic
xi=1;           % due to effective treatment


%% Initial conditions
N0=50;          % number of beds in the hospital
NSplus=r*bs*N0*(alpha*mu+tauminus)/(mu*(mu+tauplus+tauminus));
NSminus=r*bs*N0*((1-alpha)*mu+tauplus)/(mu*(mu+tauplus+tauminus));
KK=floor((N0-(NSplus+NSminus))/2);

Splus0=floor(NSplus)+KK;
Sminus0=floor(NSminus)+KK-1;
Cplus0=1;
Cminus0=1;
CTplus0=0;
CTminus0=0;
I0=0;
Rec0=0;
Nbeds=Splus0+Sminus0+Cplus0+Cminus0+CTplus0+CTminus0+I0;

%% transmission parameter
% We need to assign Ro and then calculate beta from a given Ro
Rrep0=1.3;
NCCplus = r*Nbeds*fplus*kappa*nu*(psi*(tauplus+(1-alpha)*mu)+tauminus+alpha*mu)/(mu*(mu+tauplus+tauminus)*(mu+gamma));
NCCminus = r*Nbeds*(1-fplus)*kappa*(psi*(tauplus+(1-alpha)*mu)+tauminus+alpha*mu)/(mu*(mu+tauplus+tauminus)*(mu+epsilon));
NII = r*Nbeds*(1-fplus)*epsilon/((mu+epsilon)*(rho+muI));

beta=Rrep0/(NCCplus + NCCminus + NII);

%% Simulation time and number of realizations for computer cores
Tmax=200;
tau=1;
ttime=0:tau:Tmax;
numsims=500;
ll=0;

%% Simulations
for ii=1:numsims
        ii
        ncore=2; 
    parfor i=1:ncore
    [T{i,ii},Splus{i,ii},Sminus{i,ii},Cplus{i,ii},Cminus{i,ii},CTplus{i,ii},CTminus{i,ii},...
        I{i,ii},Rec{i,ii}]=PopulateND(theta,bs,alpha,...
    r,beta,kappa,nu,xi,gamma,sigma,tauminus,tauplus,mu,psi,fplus,epsilon,muI,eta,delta,rho,q,Nbeds,...
    Splus0,Sminus0,Cplus0,Cminus0,CTplus0,CTminus0,I0,Rec0,Tmax);
    end
    for i=1:ncore
            ll=ll+1;
    SSplus(ll,1)=Splus{i,ii}(1);
    SSminus(ll,1)=Sminus{i,ii}(1);
    CCplus(ll,1)=Cplus{i,ii}(1);
    CCminus(ll,1)=Cminus{i,ii}(1);
    CCTplus(ll,1)=CTplus{i,ii}(1);
    CCTminus(ll,1)=CTminus{i,ii}(1);
    II(ll,1)=I{i,ii}(1);
    RRec(ll,1)=Rec{i,ii}(1);
    
    k=2;
    for j=2:length(T{i,ii})
        for bb=k:length(ttime)
            if ttime(bb)<=T{i,ii}(j)
                SSplus(ll,bb)=Splus{i,ii}(j);
                SSminus(ll,bb)=Sminus{i,ii}(j);
                CCplus(ll,bb)=Cplus{i,ii}(j);
                CCminus(ll,bb)=Cminus{i,ii}(j);
                CCTplus(ll,bb)=CTplus{i,ii}(j);
                CCTminus(ll,bb)=CTminus{i,ii}(j);
                II(ll,bb)=I{i,ii}(j);
                RRec(ll,bb)=Rec{i,ii}(j);
            else
                break
            end
        end
        k=bb;
    end
    end
end

%% Averaging
aveSplus=sum(SSplus,1)/size(SSplus,1);
aveSminus=sum(SSminus,1)/size(SSminus,1);
aveCplus=sum(CCplus,1)/size(CCplus,1);
aveCminus=sum(CCminus,1)/size(CCminus,1);
aveCTplus=sum(CCTplus,1)/size(CCTplus,1);
aveCTminus=sum(CCTminus,1)/size(CCTminus,1);
aveI=sum(II,1)/size(II,1);
aveRec=sum(RRec,1)/size(RRec,1);

%% Calculating daily incidence/Prevalence and its average

for j=1:numsims
for i=2:200
    Incidence(j,i)=RRec(j,i)-RRec(j,i-1);
end
end
aveInc=sum(Incidence,1)/size(Incidence,1);

PrevNT=aveCplus+aveCminus;
PrevT=+aveCTplus+aveCTminus+aveI;
Prevalence=PrevNT+PrevT;
